% This program moves a selected feature from mask1 to mask2 by using STFT 
% and its inverse transform. 
% Input: s - original speech
% Output: s1 - the modified speech
function s1 = move(s, Fs, mask1, mask2)

% This number only works when Fs = 16000, for other Fs re-calibration is
% needed to get this number
delay = 736; % delay of the auditory filter introduced by AIgram 4.0
s = [zeros(delay,1); s(1:end-delay)];

%delay = 0.046; % delay of the auditory filter introduced by AIgram 4.0
%s = [zeros(delay*Fs,1);s(1:end-delay*Fs)];

framesize = 0.010; % 5ms
wsize = round(framesize*Fs);
[S, s_length] = STFT(s, wsize); 

[nframe_stft, nband_stft] = size(S); 

reg1 = ones(size(S));
reg2 = ones(size(S));

% first transform the REG of the source in AIgram to a REG in STFT space
t_AI_1 = mask1.pos(1,:);
f_AI_1 = mask1.pos(2,:);
gain_1 = mask1.gain;
    
[t_STFT_1, f_STFT_1] = ai2stft(t_AI_1, f_AI_1, nband_stft, nframe_stft, Fs);
    
reg_xy_1 = zeros(2, length(t_STFT_1));
reg_xy_1(1,:) = t_STFT_1;
reg_xy_1(2,:) = f_STFT_1;
    
reg1 = gen_mask(reg_xy_1, gain_1, nframe_stft, nband_stft);
reg1 = reg1';

% then transform the REG of the destination in AIgram to a REG in STFT space
t_AI_2 = mask2.pos(1,:);
f_AI_2 = mask2.pos(2,:);
gain_2 = mask2.gain;
    
[t_STFT_2, f_STFT_2] = ai2stft(t_AI_2, f_AI_2, nband_stft, nframe_stft, Fs);
    
reg_xy_2 = zeros(2, length(t_STFT_2));
reg_xy_2(1,:) = t_STFT_2;
reg_xy_2(2,:) = f_STFT_2;
    
reg2 = gen_mask(reg_xy_2, gain_2, nframe_stft, nband_stft);
reg2 = reg2';


% set the frequency component above 7.4kHz to zeros
[nframe, nfreqbin] = size(reg1);
p = floor(7400/(Fs/2)*nfreqbin);
reg_final(:,p:end) = 0;

S = S.*reg1 + S.*reg1;
s1 = ISTFT(S, s_length); 
%s1 = [s1(delay*Fs+1:end); zeros(delay*Fs,1)]; % shift back
s1 = [s1(delay+1:end); zeros(delay,1)]; % shift back

% figure(); 
% subplot(131), imagesc(abs(S')); title('original');
% subplot(132), imagesc(reg_final'); title('mask'); [S1, s_length] = STFT(s1, wsize); 
% subplot(133), imagesc(abs(S1')); title('modified');