%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program generates a cube of AIgram with the three dimensions 
% being time, frequency, and snr. 
% Input: wave_file -- name of the wave file
%        exp -- id of the experiment (64 for MN04, 16 for MN16R)
% Output: AI_cube -- the cube of aigram (v4)
%   Author:      Feipeng Li revised from Marion's code
%   E-mail:      fli2@uiuc.edu
%   Time-stamp:  2007-06-05
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AI_cube = creat_AI_cube(talker, sound, exp)

wav_file = ['s_' talker '_' sound '.wav'];

global doc configs;

if ~exist([configs.temp '/' wav_file exp '.mat'])
    
    snrs = initialize_snrs(exp);

    S = load([configs.aigrampath '/mat/standard.mat']);
    
    [x, Fs] = load_wav(talker, sound);

    if isempty(x)
        AI_cube = [];
        return;
    end

        %%%%%%-------AI-gram v4.2--------%%%%%%
        speech_data.fs = Fs;
        speech_data.type = 'speechonly';
        speech_data.snr = 0;
        speech_data.signal = x;
    
        cd(configs.aigrampath);
        
        filterbank = 'erbaligned';
        if (strcmp(exp,'64')||strcmp(exp,'06sc')||strcmp(exp,'06sv'))
            
            %%1. Prepare the speech and generate the noise at 0 dB SNR like this:
            [ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn64( speech_data );
            %%2. Compute the auditory representation of the speech-only signal like this:
            [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
            %%3. Compute the noise statistics:
            [ E_ni2, E_ni ] = compute_noise_stats('mn64', filterbank, std(noise_only) );  
        
        elseif (strcmp(exp,'16')||strcmp(exp,'06wc')||strcmp(exp,'06wv'))
            
            %%1. Prepare the speech and generate the noise at 0 dB SNR like this:            
            [ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn16r( speech_data );  
            %%2. Compute the auditory representation of the speech-only signal like this:
            [ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
            %%3. Compute the noise statistics:
            [ E_ni2, E_ni ] = compute_noise_stats('mn16r', filterbank, std(noise_only) );                
        
        end     
    
    
    for i= 1:length(snrs)

        if (strcmp(exp,'64')||strcmp(exp,'06sc')||strcmp(exp,'06sv'))
            
            %%4. Scale the noise statistics to the appropriate SNR:
            E_ni2_scaled = E_ni2 * 10^(-snrs(i)/10);
            E_ni_scaled = E_ni * 10^(-snrs(i)/20);   
            %%5. Make AI-gram out of existing information:
            [aigram, cc_d, time_axis, freq_axis ] = form_image_ai0607c( level3, ...
            'mn64', filterbank, E_ni2_scaled, E_ni_scaled);   
        
        elseif (strcmp(exp,'16')||strcmp(exp,'06wc')||strcmp(exp,'06wv'))
            
            %%4. Scale the noise statistics to the appropriate SNR:
            E_ni2_scaled = E_ni2 * 10^(-snrs(i)/10);
            E_ni_scaled = E_ni * 10^(-snrs(i)/20);            
            %%5. Make AI-gram out of existing information:
            [aigram, cc_d, time_axis, freq_axis ] = form_image_ai0607c( level3, ...
            'mn16r', filterbank, E_ni2_scaled, E_ni_scaled); 
        
        end 
        
        
    
%     for i= 1:length(snrs)
%     if exp ==64
%         % Generate the speech-shaped noise by using prepare_speech_mn64()
%         speech_data.fs = Fs;
%         speech_data.signal = x;
%         speech_data.snr = snrs(i);
%         speech_data.type = 'speechonly';
%         [speech_in, speech_only, noise_only, noise_std]=prepare_speech_mn64( speech_data);
% 
%         % Generate the Ai-gram for the Speech at the designated amount of noise
%         speech_data.noise = noise_only;
%         speech_data.signal = speech_only;
%         speech_data.type = 'separate';
%         [aigram, cc_d, time_axis, freq_axis ] = scaling_cc( speech_data, 'mn64', 'nadeeka', 'erbaligned' );
%     else
%         % Generate the speech-shaped noise by using prepare_speech_mn64()
%         speech_data.fs = Fs;
%         speech_data.signal = x;
%         speech_data.snr = snrs(i);
%         speech_data.type = 'speechonly';
%         [speech_in, speech_only, noise_only, noise_std]=prepare_speech_mn16r( speech_data);
% 
%         % Generate the Ai-gram for the Speech at the designated amount of noise
%         speech_data.noise = noise_only;
%         speech_data.signal = speech_only;
%         speech_data.type = 'separate';
%         [aigram, cc_d, time_axis, freq_axis ] = scaling_cc( speech_data, 'mn16', 'nadeeka', 'erbaligned' );    
%     end
%     % problem! the noise is different each time...

    % initialize the confusion patters
        
        AI(i).snr = snrs(i);
        AI(i).cc = cc_d;
        AI(i).ccgram = aigram;
        AI(i).time_axis = time_axis;
        AI(i).freq_axis =freq_axis;

    end
    
    utter = [ talker '_' sound ];
    
    CP = [];
    
    if (strcmp(exp,'64'))
        load(configs.mn64_cp);            
        
        for i=1:length(CP_MN64)
            if strcmp(utter, CP_MN64{i}.Utterance)
                CP.table = CP_MN64{i}.Table;
                CP.snrs = SNR_MN64;
                CP.resp = Resp_MN64;
                CP.utter = CP_MN64{i}.Utterance;                
            end
        end
        
    elseif (strcmp(exp,'16'))
        load(configs.mn16r_cp);

        for i=1:length(CP_MN16R)
            if strcmp(utter, CP_MN16R{i}.Utterance)
                CP.table = CP_MN16R{i}.Table;
                CP.snrs = SNR_MN16R;
                CP.resp = Resp_MN16R;
                CP.utter = CP_MN16R{i}.Utterance;
            end
        end
    elseif strcmp(exp,'06sc')||strcmp(exp,'06sv')
        load(configs.cv06swn_cp);            
        
        for i=1:length(CP_CV06SWN)
            if strcmp(utter, CP_CV06SWN{i}.Utterance)
                CP.table = CP_CV06SWN{i}.Table;
                CP.snrs = SNR_CV06;
                CP.resp = Resp_CV06;
                CP.utter = CP_CV06SWN{i}.Utterance;                
            end
        end
    elseif strcmp(exp,'06wc')||strcmp(exp,'06wv')
        load(configs.cv06wn_cp);            
        
        for i=1:length(CP_CV06WN)
            if strcmp(utter, CP_CV06WN{i}.Utterance)
                CP.table = CP_CV06WN{i}.Table;
                CP.snrs = SNR_CV06;
                CP.resp = Resp_CV06;
                CP.utter = CP_CV06WN{i}.Utterance;                
            end
        end
    end     
    
    AI(1).CP = CP; % the confusion pattern

    
    cd(configs.syspath);

    AI_cube = AI;
    save([configs.temp '/' wav_file exp '.mat'], 'AI_cube');
    %%%% Need a lot of work %%%%
else
    load([configs.temp '/' wav_file exp '.mat']); % has all the precomputed cc in it
end
