%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program plot the AIgram 
% Input: AIgram -- the AIgram to be plotted
%        wav_file -- name of the wav file (for labeling)
%        snr -- signal-to-noise ratio for a red line (for labeling)
%        t_star -- time for the event_gram (for labeling)
%        exp -- the id of the experiment (64 or 16)        
%   Author:      Feipeng Li revised from Marion's code
%   E-mail:      fli2@uiuc.edu
%   Time-stamp:  2007-06-05
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [h_axis1]=plot_AI_gram(AIgram, wav_file, snr, t_star, exp)

fontsize = 8; 

global configs;
S = load([configs.syspath '/bin/event_gram/mat/standard.mat']);
frequencies = S.frequencies;

ccgram = AIgram.ccgram;
time_axis = AIgram.time_axis;
freq_axis = AIgram.freq_axis;

[how_many_filters, how_many_samples] = size(ccgram);
bmdist = linspace(7.35,30,how_many_filters);

imagesc(ccgram); 
colormap(flipud(gray))

set(gca,'YDir','normal')
set(gca,'FontSize',fontsize)
xlabel('Time [cs]','FontSize',fontsize)
ylabel('Frequency [kHz]','FontSize',fontsize)

name = fix_underscore(wav_file);
name = name([2:end-4]); 

%if exp == '64'  %JBA String required for exp == '64'
if strcmp(exp,'64');
    title(sprintf('AI-gram of %s at %d dB SNR',name,snr),'FontSize',fontsize+1)
else
    title(sprintf('AI-gram of %s at %d dB SNR',name,snr),'FontSize',fontsize+1)
end

set(gca,'YAxisLocation','l','YTick',linspace(1,how_many_filters,11));
set(gca,'YTickLabel',0.1*round(10*frequencies(round(linspace(1,how_many_filters,11)))/1000))
set(gca,'XTick',40*[1:floor(how_many_samples/40)]);
set(gca,'XTickLabel',10*[1:floor(how_many_samples/40)]);


% set(gca,'YAxisLocation','left', 'YTickLabel',round(bmdist(round(linspace(1,how_many_filters,11)))))
% ylabel('Distance along BM [mm]');

if (t_star~='novalue')
    start_time = min(time_axis);
    end_time = max(time_axis);
    t_star_cs = 100*t_star;
    t_star_col = round((t_star-start_time)/(end_time-start_time)*how_many_samples);
    
    hold on %JBA "^" bad char in string: 't^*' --> 't*'
    text(t_star_col-30, 30 , ['t* = ' num2str(t_star_cs) ' cs'], 'FontSize',fontsize-2,'FontWeight', 'bold')
    line([t_star_col t_star_col], [1 length(frequencies)], 'Color', 'r' ,'Linewidth', 2, 'Linestyle', '--' );
    hold off
end

% h_axis1 = gca;
% h_axis2=axes('Position',get(h_axis1,'Position'));
% set(h_axis2,'FontSize',fontsize)
% set(h_axis2,'Color','none');
% set(h_axis2,'YAxisLocation','left')
% set(h_axis2,'XTick',[]);
% set(h_axis2,'YLim',get(h_axis1,'YLim'));
% set(h_axis2,'Fontsize', fontsize);
% set(h_axis2,'YTick',get(h_axis1,'YTick'),'FontSize',fontsize);
% set(h_axis2,'YTickLabel',round(bmdist(round(linspace(1,how_many_filters,11)))))
% set(h_axis2,'YAxisLocation','left')
% ylabel('Distance along BM [mm]')
